@@ -2,16 +2,21 @@
 shapefile.py
 Provides read and write support for ESRI Shapefiles.
 author: jlawhead<at>geospatialpython.com
-date: 20110927
-version: 1.1.4
+date: 20140507
+version: 1.2.1
 Compatible with Python versions 2.4-3.x
+version changelog: Fixed u() to just return the byte sequence on exception 
 """
 
+__version__ = "1.2.1"
+
 from struct import pack, unpack, calcsize, error
 import os
 import sys
 import time
 import array
+import tempfile
+
 #
 # Constants for shape types
 NULL = 0
@@ -31,6 +36,9 @@ MULTIPATCH = 31
 
 PYTHON3 = sys.version_info[0] == 3
 
+if PYTHON3:
+    xrange = range
+
 def b(v):
     if PYTHON3:
         if isinstance(v, str):
@@ -48,15 +56,24 @@ def b(v):
 
 def u(v):
     if PYTHON3:
-        if isinstance(v, bytes):
-            # For python 3 decode bytes to str.
-            return v.decode('utf-8')
-        elif isinstance(v, str):
-            # Already str.
-            return v
-        else:
-            # Error.
-            raise Exception('Unknown input type')
+        # try/catch added 2014/05/07
+        # returned error on dbf of shapefile
+        # from www.naturalearthdata.com named
+        # "ne_110m_admin_0_countries".
+        # Just returning v as is seemed to fix
+        # the problem.  This function could
+        # be condensed further.
+        try:
+          if isinstance(v, bytes):
+              # For python 3 decode bytes to str.
+              return v.decode('utf-8')
+          elif isinstance(v, str):
+              # Already str.
+              return v
+          else:
+              # Error.
+              raise Exception('Unknown input type')
+        except: return v
     else:
         # For python 2 assume str passed in and return str.
         return v
@@ -73,6 +90,16 @@ class _Array(array.array):
     def __repr__(self):
         return str(self.tolist())
 
+def signed_area(coords):
+    """Return the signed area enclosed by a ring using the linear time
+    algorithm at http://www.cgafaq.info/wiki/Polygon_Area. A value >= 0
+    indicates a counter-clockwise oriented ring.
+    """
+    xs, ys = map(list, zip(*coords))
+    xs.append(xs[1])
+    ys.append(ys[1])
+    return sum(xs[i]*(ys[i+1]-ys[i-1]) for i in range(1, len(coords)))/2.0
+
 class _Shape:
     def __init__(self, shapeType=None):
         """Stores the geometry of the different shape types
@@ -87,6 +114,78 @@ class _Shape:
         self.shapeType = shapeType
         self.points = []
 
+    @property
+    def __geo_interface__(self):
+        if self.shapeType in [POINT, POINTM, POINTZ]:
+            return {
+            'type': 'Point',
+            'coordinates': tuple(self.points[0])
+            }
+        elif self.shapeType in [MULTIPOINT, MULTIPOINTM, MULTIPOINTZ]:
+            return {
+            'type': 'MultiPoint',
+            'coordinates': tuple([tuple(p) for p in self.points])
+            }
+        elif self.shapeType in [POLYLINE, POLYLINEM, POLYLINEZ]:
+            if len(self.parts) == 1:
+                return {
+                'type': 'LineString',
+                'coordinates': tuple([tuple(p) for p in self.points])
+                }
+            else:
+                ps = None
+                coordinates = []
+                for part in self.parts:
+                    if ps == None:
+                        ps = part
+                        continue
+                    else:
+                        coordinates.append(tuple([tuple(p) for p in self.points[ps:part]]))
+                        ps = part
+                else:
+                    coordinates.append(tuple([tuple(p) for p in self.points[part:]]))
+                return {
+                'type': 'MultiLineString',
+                'coordinates': tuple(coordinates)
+                }
+        elif self.shapeType in [POLYGON, POLYGONM, POLYGONZ]:
+            if len(self.parts) == 1:
+                return {
+                'type': 'Polygon',
+                'coordinates': (tuple([tuple(p) for p in self.points]),)
+                }
+            else:
+                ps = None
+                coordinates = []
+                for part in self.parts:
+                    if ps == None:
+                        ps = part
+                        continue
+                    else:
+                        coordinates.append(tuple([tuple(p) for p in self.points[ps:part]]))
+                        ps = part
+                else:
+                    coordinates.append(tuple([tuple(p) for p in self.points[part:]]))
+                polys = []
+                poly = [coordinates[0]]
+                for coord in coordinates[1:]:
+                    if signed_area(coord) < 0:
+                        polys.append(poly)
+                        poly = [coord]
+                    else:
+                        poly.append(coord)
+                polys.append(poly)
+                if len(polys) == 1:
+                    return {
+                    'type': 'Polygon',
+                    'coordinates': tuple(polys[0])
+                    }
+                elif len(polys) > 1:
+                    return {
+                    'type': 'MultiPolygon',
+                    'coordinates': polys
+                    }
+
 class _ShapeRecord:
     """A shape object of any type."""
     def __init__(self, shape=None, record=None):
@@ -127,7 +226,7 @@ class Reader:
         self.__dbfHdrLength = 0
         # See if a shapefile name was passed as an argument
         if len(args) > 0:
-            if type(args[0]) is type("stringTest"):
+            if is_string(args[0]):
                 self.load(args[0])
                 return
         if "shp" in kwargs.keys():
@@ -220,6 +319,8 @@ class Reader:
         record = _Shape()
         nParts = nPoints = zmin = zmax = mmin = mmax = None
         (recNum, recLength) = unpack(">2i", f.read(8))
+        # Determine the start of the next record
+        next = f.tell() + (2 * recLength)
         shapeType = unpack("<i", f.read(4))[0]
         record.shapeType = shapeType
         # For Null shapes create an empty points list for consistency
@@ -247,12 +348,12 @@ class Reader:
         if shapeType in (13,15,18,31):
             (zmin, zmax) = unpack("<2d", f.read(16))
             record.z = _Array('d', unpack("<%sd" % nPoints, f.read(nPoints * 8)))
-        # Read m extremes and values
-        if shapeType in (13,15,18,23,25,28,31):
+        # Read m extremes and values if header m values do not equal 0.0
+        if shapeType in (13,15,18,23,25,28,31) and not 0.0 in self.measure:
             (mmin, mmax) = unpack("<2d", f.read(16))
             # Measure values less than -10e38 are nodata values according to the spec
             record.m = []
-            for m in _Array('d', unpack("%sd" % nPoints, f.read(nPoints * 8))):
+            for m in _Array('d', unpack("<%sd" % nPoints, f.read(nPoints * 8))):
                 if m > -10e38:
                     record.m.append(m)
                 else:
@@ -266,6 +367,10 @@ class Reader:
         # Read a single M value
         if shapeType in (11,21):
             record.m = unpack("<d", f.read(8))
+        # Seek to the end of this record as defined by the record header because
+        # the shapefile spec doesn't require the actual content to meet the header
+        # definition.  Probably allowed for lazy feature deletion. 
+        f.seek(next)
         return record
 
     def __shapeIndex(self, i=None):
@@ -295,21 +400,38 @@ class Reader:
         i = self.__restrictIndex(i)
         offset = self.__shapeIndex(i)
         if not offset:
-            # Shx index not available so use the full list.
-            shapes = self.shapes()
-            return shapes[i]
+            # Shx index not available so iterate the full list.
+            for j,k in enumerate(self.iterShapes()):
+                if j == i:
+                    return k
         shp.seek(offset)
         return self.__shape()
 
     def shapes(self):
         """Returns all shapes in a shapefile."""
         shp = self.__getFileObj(self.shp)
+        # Found shapefiles which report incorrect
+        # shp file length in the header. Can't trust
+        # that so we seek to the end of the file
+        # and figure it out.
+        shp.seek(0,2)
+        self.shpLength = shp.tell()
         shp.seek(100)
         shapes = []
         while shp.tell() < self.shpLength:
             shapes.append(self.__shape())
         return shapes
 
+    def iterShapes(self):
+        """Serves up shapes in a shapefile as an iterator. Useful
+        for handling large shapefiles."""
+        shp = self.__getFileObj(self.shp)
+        shp.seek(0,2)
+        self.shpLength = shp.tell()
+        shp.seek(100)
+        while shp.tell() < self.shpLength:
+            yield self.__shape()    
+
     def __dbfHeaderLength(self):
         """Retrieves the header length of a dbf file header."""
         if not self.__dbfHdrLength:
@@ -415,12 +537,23 @@ class Reader:
                 records.append(r)
         return records
 
+    def iterRecords(self):
+        """Serves up records in a dbf file as an iterator.
+        Useful for large shapefiles or dbf files."""
+        if not self.numRecords:
+            self.__dbfHeader()
+        f = self.__getFileObj(self.dbf)
+        f.seek(self.__dbfHeaderLength())
+        for i in xrange(self.numRecords):
+            r = self.__record()
+            if r:
+                yield r
+
     def shapeRecord(self, i=0):
         """Returns a combination geometry and attribute record for the
         supplied record index."""
         i = self.__restrictIndex(i)
-        return _ShapeRecord(shape=self.shape(i),
-                                                        record=self.record(i))
+        return _ShapeRecord(shape=self.shape(i), record=self.record(i))
 
     def shapeRecords(self):
         """Returns a list of combination geometry/attribute records for
@@ -638,6 +771,8 @@ class Writer:
             recNum += 1
             start = f.tell()
             # Shape Type
+            if self.shapeType != 31:
+                s.shapeType = self.shapeType
             f.write(pack("<i", s.shapeType))
             # All shape types capable of having a bounding box
             if s.shapeType in (3,5,8,13,15,18,23,25,28,31):
@@ -674,13 +809,19 @@ class Writer:
                 except error:
                     raise ShapefileException("Failed to write elevation extremes for record %s. Expected floats." % recNum)
                 try:
-                    [f.write(pack("<d", p[2])) for p in s.points]
+                    if hasattr(s,"z"):
+                        f.write(pack("<%sd" % len(s.z), *s.z))
+                    else:
+                        [f.write(pack("<d", p[2])) for p in s.points]  
                 except error:
                     raise ShapefileException("Failed to write elevation values for record %s. Expected floats." % recNum)
             # Write m extremes and values
-            if s.shapeType in (23,25,31):
+            if s.shapeType in (13,15,18,23,25,28,31):
                 try:
-                    f.write(pack("<2d", *self.__mbox([s])))
+                    if hasattr(s,"m"):
+                        f.write(pack("<%sd" % len(s.m), *s.m))
+                    else:
+                        f.write(pack("<2d", *self.__mbox([s])))
                 except error:
                     raise ShapefileException("Failed to write measure extremes for record %s. Expected floats" % recNum)
                 try:
@@ -695,16 +836,36 @@ class Writer:
                     raise ShapefileException("Failed to write point for record %s. Expected floats." % recNum)
             # Write a single Z value
             if s.shapeType == 11:
-                try:
-                    f.write(pack("<1d", s.points[0][2]))
-                except error:
-                    raise ShapefileException("Failed to write elevation value for record %s. Expected floats." % recNum)
+                if hasattr(s, "z"):
+                    try:
+                        if not s.z:
+                            s.z = (0,)    
+                        f.write(pack("<d", s.z[0]))
+                    except error:
+                        raise ShapefileException("Failed to write elevation value for record %s. Expected floats." % recNum)
+                else:
+                    try:
+                        if len(s.points[0])<3:
+                            s.points[0].append(0)
+                        f.write(pack("<d", s.points[0][2]))
+                    except error:
+                        raise ShapefileException("Failed to write elevation value for record %s. Expected floats." % recNum)
             # Write a single M value
             if s.shapeType in (11,21):
-                try:
-                    f.write(pack("<1d", s.points[0][3]))
-                except error:
-                    raise ShapefileException("Failed to write measure value for record %s. Expected floats." % recNum)
+                if hasattr(s, "m"):
+                    try:
+                        if not s.m:
+                            s.m = (0,) 
+                        f.write(pack("<1d", s.m[0]))
+                    except error:
+                        raise ShapefileException("Failed to write measure value for record %s. Expected floats." % recNum)    
+                else:                                
+                    try:
+                        if len(s.points[0])<4:
+                            s.points[0].append(0)
+                        f.write(pack("<1d", s.points[0][3]))
+                    except error:
+                        raise ShapefileException("Failed to write measure value for record %s. Expected floats." % recNum)
             # Finalize record length as 16-bit words
             finish = f.tell()
             length = (finish - start) // 2
@@ -766,6 +927,11 @@ class Writer:
         polyShape = _Shape(shapeType)
         polyShape.parts = []
         polyShape.points = []
+        # Make sure polygons are closed
+        if shapeType in (5,15,25,31):
+            for part in parts:
+                    if part[0] != part[-1]:
+                        part.append(part[0])
         for part in parts:
             polyShape.parts.append(len(polyShape.points))
             for point in part:
@@ -805,10 +971,10 @@ class Writer:
             for field in self.fields:
                 if field[0] in recordDict:
                     val = recordDict[field[0]]
-                    if val:
-                        record.append(val)
-                    else:
+                    if val is None:
                         record.append("")
+                    else:
+                        record.append(val)
         if record:
             self.records.append(record)
 
@@ -850,22 +1016,33 @@ class Writer:
     def save(self, target=None, shp=None, shx=None, dbf=None):
         """Save the shapefile data to three files or
         three file-like objects. SHP and DBF files can also
-        be written exclusively using saveShp, saveShx, and saveDbf respectively."""
-        # TODO: Create a unique filename for target if None.
+        be written exclusively using saveShp, saveShx, and saveDbf respectively.
+        If target is specified but not shp,shx, or dbf then the target path and
+        file name are used.  If no options or specified, a unique base file name
+        is generated to save the files and the base file name is returned as a 
+        string. 
+        """
+        # Create a unique file name if one is not defined
         if shp:
             self.saveShp(shp)
         if shx:
             self.saveShx(shx)
         if dbf:
             self.saveDbf(dbf)
-        elif target:
+        elif not shp and not shx and not dbf:
+            generated = False
+            if not target:
+                temp = tempfile.NamedTemporaryFile(prefix="shapefile_",dir=os.getcwd())
+                target = temp.name
+                generated = True         
             self.saveShp(target)
             self.shp.close()
             self.saveShx(target)
             self.shx.close()
             self.saveDbf(target)
             self.dbf.close()
-
+            if generated:
+                return target
 class Editor(Writer):
     def __init__(self, shapefile=None, shapeType=POINT, autoBalance=1):
         self.autoBalance = autoBalance
@@ -991,7 +1168,7 @@ def test():
 
 if __name__ == "__main__":
     """
-    Doctests are contained in the module 'pyshp_usage.py'. This library was developed
+    Doctests are contained in the file 'README.txt'. This library was originally developed
     using Python 2.3. Python 2.4 and above have some excellent improvements in the built-in
     testing libraries but for now unit testing is done using what's available in
     2.3.
